@@ -21,13 +21,22 @@ from heat.engine import support
 
 
 class DesignateDomain(resource.Resource):
-    """Heat Template Resource for Designate Domain."""
+    """Heat Template Resource for Designate Domain.
+
+    Designate provides DNS-as-a-Service services for OpenStack. So, domain
+    is a realm with an identification string, unique in DNS.
+    """
 
     support_status = support.SupportStatus(
-        version='5.0.0')
+        status=support.DEPRECATED,
+        version='8.0.0',
+        message=_('Use OS::Designate::Zone instead.'),
+        previous_status=support.SupportStatus(version='5.0.0'))
 
     entity = 'domains'
 
+    default_client_name = 'designate'
+
     PROPERTIES = (
         NAME, TTL, DESCRIPTION, EMAIL
     ) = (
@@ -78,10 +87,6 @@ class DesignateDomain(resource.Resource):
         ),
     }
 
-    default_client_name = 'designate'
-
-    entity = 'domains'
-
     def handle_create(self):
         args = dict((k, v) for k, v in six.iteritems(self.properties) if v)
         domain = self.client_plugin().domain_create(**args)
@@ -105,6 +110,8 @@ class DesignateDomain(resource.Resource):
             self.client_plugin().domain_update(**args)
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         if name == self.SERIAL:
             domain = self.client().domains.get(self.resource_id)
             return domain.serial
@@ -114,6 +121,14 @@ class DesignateDomain(resource.Resource):
     def _show_resource(self):
         return dict(self.client().domains.get(self.resource_id).items())
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        domain_reality = {}
+
+        for key in self.PROPERTIES:
+            domain_reality.update({key: resource_data.get(key)})
+
+        return domain_reality
+
 
 def resource_mapping():
     return {
