@@ -12,17 +12,14 @@
 #    under the License.
 
 import copy
-import uuid
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 import six
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LE
 from heat.engine import attributes
 from heat.engine.clients import progress
 from heat.engine import constraints
@@ -32,18 +29,18 @@ from heat.engine.resources.openstack.neutron import port as neutron_port
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine.resources.openstack.nova import server_network_mixin
 from heat.engine.resources import scheduler_hints as sh
-from heat.engine.resources import stack_user
+from heat.engine.resources import server_base
 from heat.engine import support
 from heat.engine import translation
 from heat.rpc import api as rpc_api
 
 cfg.CONF.import_opt('default_software_config_transport', 'heat.common.config')
-cfg.CONF.import_opt('max_server_name_length', 'heat.common.config')
+cfg.CONF.import_opt('default_user_data_format', 'heat.common.config')
 
 LOG = logging.getLogger(__name__)
 
 
-class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
+class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
              server_network_mixin.ServerNetworkMixin):
     """A resource for managing Nova instances.
 
@@ -57,14 +54,16 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         ADMIN_USER, AVAILABILITY_ZONE, SECURITY_GROUPS, NETWORKS,
         SCHEDULER_HINTS, METADATA, USER_DATA_FORMAT, USER_DATA,
         RESERVATION_ID, CONFIG_DRIVE, DISK_CONFIG, PERSONALITY,
-        ADMIN_PASS, SOFTWARE_CONFIG_TRANSPORT, USER_DATA_UPDATE_POLICY
+        ADMIN_PASS, SOFTWARE_CONFIG_TRANSPORT, USER_DATA_UPDATE_POLICY,
+        TAGS
     ) = (
         'name', 'image', 'block_device_mapping', 'block_device_mapping_v2',
         'flavor', 'flavor_update_policy', 'image_update_policy', 'key_name',
         'admin_user', 'availability_zone', 'security_groups', 'networks',
         'scheduler_hints', 'metadata', 'user_data_format', 'user_data',
         'reservation_id', 'config_drive', 'diskConfig', 'personality',
-        'admin_pass', 'software_config_transport', 'user_data_update_policy'
+        'admin_pass', 'software_config_transport', 'user_data_update_policy',
+        'tags'
     )
 
     _BLOCK_DEVICE_MAPPING_KEYS = (
@@ -91,6 +90,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         BLOCK_DEVICE_MAPPING_BOOT_INDEX,
         BLOCK_DEVICE_MAPPING_VOLUME_SIZE,
         BLOCK_DEVICE_MAPPING_DELETE_ON_TERM,
+        BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE,
+        BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT,
     ) = (
         'device_name',
         'volume_id',
@@ -103,6 +104,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         'boot_index',
         'volume_size',
         'delete_on_termination',
+        'ephemeral_size',
+        'ephemeral_format'
     )
 
     _NETWORK_KEYS = (
@@ -127,10 +130,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
     ATTRIBUTES = (
         NAME_ATTR, ADDRESSES, NETWORKS_ATTR, FIRST_ADDRESS,
-        INSTANCE_NAME, ACCESSIPV4, ACCESSIPV6, CONSOLE_URLS,
+        INSTANCE_NAME, ACCESSIPV4, ACCESSIPV6, CONSOLE_URLS, TAGS_ATTR
     ) = (
         'name', 'addresses', 'networks', 'first_address',
-        'instance_name', 'accessIPv4', 'accessIPv6', 'console_urls',
+        'instance_name', 'accessIPv4', 'accessIPv6', 'console_urls', 'tags'
     )
 
     # valid image Status
@@ -250,6 +253,24 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                         properties.Schema.INTEGER,
                         _('The size of the swap, in MB.')
                     ),
+                    BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE: properties.Schema(
+                        properties.Schema.INTEGER,
+                        _('The size of the local ephemeral block device, '
+                          'in GB.'),
+                        support_status=support.SupportStatus(version='8.0.0'),
+                        constraints=[constraints.Range(min=1)]
+                    ),
+                    BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT: properties.Schema(
+                        properties.Schema.STRING,
+                        _('The format of the local ephemeral block device. '
+                          'If no format is specified, uses default value, '
+                          'defined in nova configuration file.'),
+                        constraints=[
+                            constraints.AllowedValues(['ext2', 'ext3', 'ext4',
+                                                       'xfs', 'ntfs'])
+                        ],
+                        support_status=support.SupportStatus(version='8.0.0')
+                    ),
                     BLOCK_DEVICE_MAPPING_DEVICE_TYPE: properties.Schema(
                         properties.Schema.STRING,
                         _('Device type: at the moment we can make distinction '
@@ -433,7 +454,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
               'keys and values must be 255 characters or less. Non-string '
               'values will be serialized to JSON (and the serialized '
               'string must be 255 characters or less).'),
-            update_allowed=True
+            update_allowed=True,
+            default={}
         ),
         USER_DATA_FORMAT: properties.Schema(
             properties.Schema.STRING,
@@ -444,7 +466,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
               'For SOFTWARE_CONFIG user_data is bundled as part of the '
               'software config data, and metadata is derived from any '
               'associated SoftwareDeployment resources.'),
-            default=HEAT_CFNTOOLS,
+            default=cfg.CONF.default_user_data_format,
             constraints=[
                 constraints.AllowedValues(_SOFTWARE_CONFIG_FORMATS),
             ]
@@ -510,6 +532,13 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             _('The administrator password for the server.'),
             update_allowed=True
         ),
+        TAGS: properties.Schema(
+            properties.Schema.LIST,
+            _('Server tags. Supported since client version 2.26.'),
+            support_status=support.SupportStatus(version='8.0.0'),
+            schema=properties.Schema(properties.Schema.STRING),
+            update_allowed=True
+        )
     }
 
     attributes_schema = {
@@ -576,14 +605,15 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             support_status=support.SupportStatus(version='2015.1'),
             type=attributes.Schema.MAP
         ),
+        TAGS_ATTR: attributes.Schema(
+            _('Tags from the server. Supported since client version 2.26.'),
+            support_status=support.SupportStatus(version='8.0.0'),
+            type=attributes.Schema.LIST
+        )
     }
 
-    physical_resource_name_limit = cfg.CONF.max_server_name_length
-
     default_client_name = 'nova'
 
-    entity = 'servers'
-
     def translation_rules(self, props):
         rules = [
             translation.TranslationRule(
@@ -654,146 +684,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         super(Server, self).__init__(name, json_snippet, stack)
         if self.user_data_software_config():
             self._register_access_key()
-
-    def _server_name(self):
-        name = self.properties[self.NAME]
-        if name:
-            return name
-
-        return self.physical_resource_name()
+        self.default_collectors = ['ec2']
 
     def _config_drive(self):
         # This method is overridden by the derived CloudServer resource
         return self.properties[self.CONFIG_DRIVE]
 
-    def _populate_deployments_metadata(self, meta, props):
-        meta['deployments'] = meta.get('deployments', [])
-        meta['os-collect-config'] = meta.get('os-collect-config', {})
-        occ = meta['os-collect-config']
-        collectors = ['ec2']
-        occ['collectors'] = collectors
-
-        # set existing values to None to override any boot-time config
-        occ_keys = ('heat', 'zaqar', 'cfn', 'request')
-        for occ_key in occ_keys:
-            if occ_key not in occ:
-                continue
-            existing = occ[occ_key]
-            for k in existing:
-                existing[k] = None
-
-        if self.transport_poll_server_heat(props):
-            occ.update({'heat': {
-                'user_id': self._get_user_id(),
-                'password': self.password,
-                'auth_url': self.context.auth_url,
-                'project_id': self.stack.stack_user_project_id,
-                'stack_id': self.stack.identifier().stack_path(),
-                'resource_name': self.name}})
-            collectors.append('heat')
-
-        elif self.transport_zaqar_message(props):
-            queue_id = self.physical_resource_name()
-            self.data_set('metadata_queue_id', queue_id)
-            occ.update({'zaqar': {
-                'user_id': self._get_user_id(),
-                'password': self.password,
-                'auth_url': self.context.auth_url,
-                'project_id': self.stack.stack_user_project_id,
-                'queue_id': queue_id}})
-            collectors.append('zaqar')
-
-        elif self.transport_poll_server_cfn(props):
-            heat_client_plugin = self.stack.clients.client_plugin('heat')
-            config_url = heat_client_plugin.get_cfn_metadata_server_url()
-            occ.update({'cfn': {
-                'metadata_url': config_url,
-                'access_key_id': self.access_key,
-                'secret_access_key': self.secret_key,
-                'stack_name': self.stack.name,
-                'path': '%s.Metadata' % self.name}})
-            collectors.append('cfn')
-
-        elif self.transport_poll_temp_url(props):
-            container = self.physical_resource_name()
-            object_name = self.data().get('metadata_object_name')
-            if not object_name:
-                object_name = str(uuid.uuid4())
-
-            self.client('swift').put_container(container)
-
-            url = self.client_plugin('swift').get_temp_url(
-                container, object_name, method='GET')
-            put_url = self.client_plugin('swift').get_temp_url(
-                container, object_name)
-            self.data_set('metadata_put_url', put_url)
-            self.data_set('metadata_object_name', object_name)
-
-            collectors.append('request')
-            occ.update({'request': {
-                'metadata_url': url}})
-
-        collectors.append('local')
-        self.metadata_set(meta)
-
-        # push replacement polling config to any existing push-based sources
-        queue_id = self.data().get('metadata_queue_id')
-        if queue_id:
-            zaqar_plugin = self.client_plugin('zaqar')
-            zaqar = zaqar_plugin.create_for_tenant(
-                self.stack.stack_user_project_id, self._user_token())
-            queue = zaqar.queue(queue_id)
-            queue.post({'body': meta, 'ttl': zaqar_plugin.DEFAULT_TTL})
-
-        object_name = self.data().get('metadata_object_name')
-        if object_name:
-            container = self.physical_resource_name()
-            self.client('swift').put_object(
-                container, object_name, jsonutils.dumps(meta))
-
-    def _register_access_key(self):
-        """Access is limited to this resource, which created the keypair."""
-        def access_allowed(resource_name):
-            return resource_name == self.name
-
-        if self.access_key is not None:
-            self.stack.register_access_allowed_handler(
-                self.access_key, access_allowed)
-        if self._get_user_id() is not None:
-            self.stack.register_access_allowed_handler(
-                self._get_user_id(), access_allowed)
-
-    def _create_transport_credentials(self, props):
-        if self.transport_poll_server_cfn(props):
-            self._create_user()
-            self._create_keypair()
-
-        elif (self.transport_poll_server_heat(props) or
-              self.transport_zaqar_message(props)):
-            self.password = uuid.uuid4().hex
-            self._create_user()
-
-        self._register_access_key()
-
-    @property
-    def access_key(self):
-        return self.data().get('access_key')
-
-    @property
-    def secret_key(self):
-        return self.data().get('secret_key')
-
-    @property
-    def password(self):
-        return self.data().get('password')
-
-    @password.setter
-    def password(self, password):
-        if password is None:
-            self.data_delete('password')
-        else:
-            self.data_set('password', password, True)
-
     def user_data_raw(self):
         return self.properties[self.USER_DATA_FORMAT] == self.RAW
 
@@ -801,22 +697,6 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         return self.properties[
             self.USER_DATA_FORMAT] == self.SOFTWARE_CONFIG
 
-    def transport_poll_server_cfn(self, props):
-        return props[
-            self.SOFTWARE_CONFIG_TRANSPORT] == self.POLL_SERVER_CFN
-
-    def transport_poll_server_heat(self, props):
-        return props[
-            self.SOFTWARE_CONFIG_TRANSPORT] == self.POLL_SERVER_HEAT
-
-    def transport_poll_temp_url(self, props):
-        return props[
-            self.SOFTWARE_CONFIG_TRANSPORT] == self.POLL_TEMP_URL
-
-    def transport_zaqar_message(self, props):
-        return props[
-            self.SOFTWARE_CONFIG_TRANSPORT] == self.ZAQAR_MESSAGE
-
     def get_software_config(self, ud_content):
         try:
             sc = self.rpc_client().show_software_config(
@@ -850,7 +730,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
         availability_zone = self.properties[self.AVAILABILITY_ZONE]
         instance_meta = self.properties[self.METADATA]
-        if instance_meta is not None:
+        if instance_meta:
             instance_meta = self.client_plugin().meta_serialize(
                 instance_meta)
 
@@ -902,12 +782,19 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
     def check_create_complete(self, server_id):
         check = self.client_plugin()._check_active(server_id)
         if check:
+            if self.properties[self.TAGS]:
+                self._update_server_tags(self.properties[self.TAGS])
             self.store_external_ports()
             # Addresses binds to server not immediately, so we need to wait
             # until server is created and after that associate floating ip.
             self.floating_ips_nova_associate()
         return check
 
+    def _update_server_tags(self, tags):
+        server = self.client().servers.get(self.resource_id)
+        self.client(version=self.client_plugin().V2_26
+                    ).servers.set_tags(server, tags)
+
     def floating_ips_nova_associate(self):
         # If there is no neutron used, floating_ip still unassociated,
         # so need associate it with nova.
@@ -923,6 +810,82 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         checks = [{'attr': 'status', 'expected': 'ACTIVE', 'current': status}]
         self._verify_check_conditions(checks)
 
+    def get_live_resource_data(self):
+        try:
+            server = self.client().servers.get(self.resource_id)
+            server_data = server.to_dict()
+            active = self.client_plugin()._check_active(server)
+            if not active:
+                # There is no difference what error raised, because update
+                # method of resource just silently log it as warning.
+                raise exception.Error(_('Server %s is not '
+                                        'in ACTIVE state') % self.name)
+        except Exception as ex:
+            if self.client_plugin().is_not_found(ex):
+                raise exception.EntityNotFound(entity='Resource',
+                                               name=self.name)
+            raise
+
+        try:
+            tag_server = self.client(
+                version=self.client_plugin().V2_26
+            ).server.get(self.resource_id)
+        except Exception as ex:
+            LOG.warning('Cannot resolve tags for observe reality in case of '
+                        'unsupported minimal version tag support client')
+        else:
+            server_data['tags'] = tag_server.tag_list()
+        return server, server_data
+
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        server, server_data = resource_data
+        result = {
+            # there's a risk that flavor id will be int type, so cast to str
+            self.FLAVOR: six.text_type(server_data.get(self.FLAVOR)['id']),
+            self.IMAGE: six.text_type(server_data.get(self.IMAGE)['id']),
+            self.NAME: server_data.get(self.NAME),
+            self.METADATA: server_data.get(self.METADATA),
+            self.NETWORKS: self._get_live_networks(server, resource_properties)
+        }
+        if 'tags' in server_data:
+            result.update({self.TAGS: server_data['tags']})
+        return result
+
+    def _get_live_networks(self, server, props):
+        reality_nets = self._add_port_for_address(server,
+                                                  extend_networks=False)
+        reality_net_ids = {}
+        for net_key in reality_nets:
+            try:
+                net_id = self.client_plugin().get_net_id_by_label(net_key)
+            except (exception.EntityNotFound,
+                    exception.PhysicalResourceNameAmbiguity):
+                net_id = None
+            if net_id:
+                reality_net_ids[net_id] = reality_nets.get(net_key)
+
+        resource_nets = props.get(self.NETWORKS)
+
+        result_nets = []
+        for net in resource_nets or []:
+            net_id = self._get_network_id(net)
+            if reality_net_ids.get(net_id):
+                for idx, address in enumerate(reality_net_ids.get(net_id)):
+                    if address['addr'] == net[self.NETWORK_FIXED_IP]:
+                        result_nets.append(net)
+                        reality_net_ids.get(net_id).pop(idx)
+                        break
+
+        for key, value in six.iteritems(reality_nets):
+            for address in reality_nets[key]:
+                new_net = {self.NETWORK_ID: key,
+                           self.NETWORK_FIXED_IP: address['addr']}
+                if address['port'] not in [port['id']
+                                           for port in self._data_get_ports()]:
+                    new_net.update({self.NETWORK_PORT: address['port']})
+                result_nets.append(new_net)
+        return result_nets
+
     @classmethod
     def _build_block_device_mapping(cls, bdm):
         if not bdm:
@@ -995,6 +958,22 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     'volume_size': mapping.get(
                         cls.BLOCK_DEVICE_MAPPING_SWAP_SIZE),
                 }
+            elif (mapping.get(cls.BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE) or
+                  mapping.get(cls.BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT)):
+                bmd_dict = {
+                    'source_type': 'blank',
+                    'destination_type': 'local',
+                    'boot_index': -1,
+                    'delete_on_termination': True
+                }
+                ephemeral_size = mapping.get(
+                    cls.BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE)
+                if ephemeral_size:
+                    bmd_dict.update({'volume_size': ephemeral_size})
+                ephemeral_format = mapping.get(
+                    cls.BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT)
+                if ephemeral_format:
+                    bmd_dict.update({'guest_format': ephemeral_format})
 
             # NOTE(prazumovsky): In case of server doesn't take empty value of
             # device name, need to escape from such situation.
@@ -1017,12 +996,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
         return bdm_v2_list
 
-    def _add_port_for_address(self, server):
+    def _add_port_for_address(self, server, extend_networks=True):
         """Method adds port id to list of addresses.
 
         This method is used only for resolving attributes.
         """
-        nets = copy.deepcopy(server.addresses)
+        nets = copy.deepcopy(server.addresses) or {}
         ifaces = server.interface_list()
         ip_mac_mapping_on_port_id = dict(((iface.fixed_ips[0]['ip_address'],
                                            iface.mac_addr), iface.port_id)
@@ -1031,7 +1010,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             for addr in nets[net_name]:
                 addr['port'] = ip_mac_mapping_on_port_id.get(
                     (addr['addr'], addr['OS-EXT-IPS-MAC:mac_addr']))
-        return self._extend_networks(nets)
+        if extend_networks:
+            return self._extend_networks(nets)
+        else:
+            return nets
 
     def _extend_networks(self, networks):
         """Method adds same networks with replaced name on network id.
@@ -1050,6 +1032,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         return nets
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         if name == self.FIRST_ADDRESS:
             return self.client_plugin().server_to_ipaddress(
                 self.resource_id) or ''
@@ -1072,6 +1056,13 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             return server.accessIPv6
         if name == self.CONSOLE_URLS:
             return self.client_plugin('nova').get_console_urls(server)
+        if name == self.TAGS_ATTR:
+            try:
+                cv = self.client(
+                    version=self.client_plugin().V2_26)
+                return cv.servers.tag_list(server)
+            except exception.InvalidServiceVersion:
+                return None
 
     def add_dependencies(self, deps):
         super(Server, self).add_dependencies(deps)
@@ -1107,8 +1098,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                         deps += (self, res)
                         break
 
-    def _update_flavor(self, prop_diff):
-        flavor = prop_diff[self.FLAVOR]
+    def _update_flavor(self, after_props):
+        flavor = after_props[self.FLAVOR]
         handler_args = checker_args = {'args': (flavor,)}
         prg_resize = progress.ServerUpdateProgress(self.resource_id,
                                                    'resize',
@@ -1118,23 +1109,19 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                                                    'verify_resize')
         return prg_resize, prg_verify
 
-    def _update_image(self, prop_diff):
-        image_update_policy = (
-            prop_diff.get(self.IMAGE_UPDATE_POLICY) or
-            self.properties[self.IMAGE_UPDATE_POLICY])
-
-        instance_meta = prop_diff.get(self.METADATA,
-                                      self.properties[self.METADATA])
+    def _update_image(self, after_props):
+        image_update_policy = after_props[self.IMAGE_UPDATE_POLICY]
 
+        instance_meta = after_props[self.METADATA]
         if instance_meta is not None:
-            instance_meta = self.client_plugin().meta_serialize(instance_meta)
-        personality_files = self.properties[self.PERSONALITY]
+            instance_meta = self.client_plugin().meta_serialize(
+                instance_meta)
+        personality_files = after_props[self.PERSONALITY]
 
-        image = prop_diff[self.IMAGE]
+        image = after_props[self.IMAGE]
         preserve_ephemeral = (
             image_update_policy == 'REBUILD_PRESERVE_EPHEMERAL')
-        password = (prop_diff.get(self.ADMIN_PASS) or
-                    self.properties[self.ADMIN_PASS])
+        password = after_props[self.ADMIN_PASS]
         kwargs = {'password': password,
                   'preserve_ephemeral': preserve_ephemeral,
                   'meta': instance_meta,
@@ -1145,11 +1132,11 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                                                            'kwargs': kwargs})
         return prg
 
-    def _update_networks(self, server, prop_diff):
+    def _update_networks(self, server, after_props):
         updaters = []
-        new_networks = prop_diff.get(self.NETWORKS)
+        new_networks = after_props[self.NETWORKS]
         old_networks = self.properties[self.NETWORKS]
-        security_groups = self.properties[self.SECURITY_GROUPS]
+        security_groups = after_props[self.SECURITY_GROUPS]
 
         if not server:
             server = self.client().servers.get(self.resource_id)
@@ -1198,85 +1185,52 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 before_props.get(self.USER_DATA_UPDATE_POLICY))
             return ud_update_policy == 'REPLACE'
 
-    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if tmpl_diff.metadata_changed():
-            # If SOFTWARE_CONFIG user_data_format is enabled we require
-            # the "deployments" and "os-collect-config" keys for Deployment
-            # polling.  We can attempt to merge the occ data, but any
-            # metadata update containing deployments will be discarded.
-            new_md = json_snippet.metadata()
-            if self.user_data_software_config():
-                metadata = self.metadata_get(True) or {}
-                new_occ_md = new_md.get('os-collect-config', {})
-                occ_md = metadata.get('os-collect-config', {})
-                occ_md.update(new_occ_md)
-                new_md['os-collect-config'] = occ_md
-                deployment_md = metadata.get('deployments', [])
-                new_md['deployments'] = deployment_md
-            self.metadata_set(new_md)
+    def needs_replace_failed(self):
+        if not self.resource_id:
+            return True
 
-        updaters = []
+        with self.client_plugin().ignore_not_found:
+            server = self.client().servers.get(self.resource_id)
+            return server.status in ('ERROR', 'DELETED', 'SOFT_DELETED')
+
+        return True
+
+    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
+        updaters = super(Server, self).handle_update(
+            json_snippet,
+            tmpl_diff,
+            prop_diff)
         server = None
 
-        if self.METADATA in prop_diff:
-            server = self.client_plugin().get_server(self.resource_id)
-            self.client_plugin().meta_update(server,
-                                             prop_diff[self.METADATA])
+        after_props = json_snippet.properties(self.properties_schema,
+                                              self.context)
+
+        if self.TAGS in prop_diff:
+            self._update_server_tags(after_props[self.TAGS] or [])
 
         if self.FLAVOR in prop_diff:
-            updaters.extend(self._update_flavor(prop_diff))
+            updaters.extend(self._update_flavor(after_props))
 
         if self.IMAGE in prop_diff:
-            updaters.append(self._update_image(prop_diff))
+            updaters.append(self._update_image(after_props))
         elif self.ADMIN_PASS in prop_diff:
             if not server:
                 server = self.client_plugin().get_server(self.resource_id)
-            server.change_password(prop_diff[self.ADMIN_PASS])
+            server.change_password(after_props[self.ADMIN_PASS])
 
         if self.NAME in prop_diff:
             if not server:
                 server = self.client_plugin().get_server(self.resource_id)
-            self.client_plugin().rename(server, prop_diff[self.NAME])
+            self.client_plugin().rename(server, after_props[self.NAME])
 
         if self.NETWORKS in prop_diff:
-            updaters.extend(self._update_networks(server, prop_diff))
-
-        if self.SOFTWARE_CONFIG_TRANSPORT in prop_diff:
-            self._update_software_config_transport(prop_diff)
+            updaters.extend(self._update_networks(server, after_props))
 
         # NOTE(pas-ha) optimization is possible (starting first task
         # right away), but we'd rather not, as this method already might
         # have called several APIs
         return updaters
 
-    def _update_software_config_transport(self, prop_diff):
-        if not self.user_data_software_config():
-            return
-        try:
-            metadata = self.metadata_get(True) or {}
-            self._create_transport_credentials(prop_diff)
-            self._populate_deployments_metadata(metadata, prop_diff)
-            # push new metadata to all sources by creating a dummy
-            # deployment
-            sc = self.rpc_client().create_software_config(
-                self.context, 'ignored', 'ignored', '')
-            sd = self.rpc_client().create_software_deployment(
-                self.context, self.resource_id, sc['id'])
-            self.rpc_client().delete_software_deployment(
-                self.context, sd['id'])
-            self.rpc_client().delete_software_config(
-                self.context, sc['id'])
-        except Exception:
-            # Updating the software config transport is on a best-effort
-            # basis as any raised exception here would result in the resource
-            # going into an ERROR state, which will be replaced on the next
-            # stack update. This is not desirable for a server. The old
-            # transport will continue to work, and the new transport may work
-            # despite exceptions in the above block.
-            LOG.exception(
-                _LE('Error while updating software config transport')
-            )
-
     def check_update_complete(self, updaters):
         """Push all updaters to completion in list order."""
         for prg in updaters:
@@ -1295,28 +1249,6 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             self.store_external_ports()
         return status
 
-    def metadata_update(self, new_metadata=None):
-        """Refresh the metadata if new_metadata is None."""
-        if new_metadata is None:
-            # Re-resolve the template metadata and merge it with the
-            # current resource metadata.  This is necessary because the
-            # attributes referenced in the template metadata may change
-            # and the resource itself adds keys to the metadata which
-            # are not specified in the template (e.g the deployments data)
-            meta = self.metadata_get(refresh=True) or {}
-            tmpl_meta = self.t.metadata()
-            meta.update(tmpl_meta)
-            self.metadata_set(meta)
-
-    @staticmethod
-    def _check_maximum(count, maximum, msg):
-        """Check a count against a maximum.
-
-        Unless maximum is -1 which indicates that there is no limit.
-        """
-        if maximum != -1 and count > maximum:
-            raise exception.StackValidationFailed(message=msg)
-
     def _validate_block_device_mapping(self):
 
         # either volume_id or snapshot_id needs to be specified, but not both
@@ -1349,19 +1281,27 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             snapshot_id = mapping.get(self.BLOCK_DEVICE_MAPPING_SNAPSHOT_ID)
             image_id = mapping.get(self.BLOCK_DEVICE_MAPPING_IMAGE)
             swap_size = mapping.get(self.BLOCK_DEVICE_MAPPING_SWAP_SIZE)
+            ephemeral = (mapping.get(
+                self.BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE) or mapping.get(
+                self.BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT))
 
-            property_tuple = (volume_id, snapshot_id, image_id, swap_size)
+            property_tuple = (volume_id, snapshot_id, image_id, swap_size,
+                              ephemeral)
 
-            if property_tuple.count(None) < 3:
+            if property_tuple.count(None) < 4:
                 raise exception.ResourcePropertyConflict(
                     self.BLOCK_DEVICE_MAPPING_VOLUME_ID,
                     self.BLOCK_DEVICE_MAPPING_SNAPSHOT_ID,
                     self.BLOCK_DEVICE_MAPPING_IMAGE,
-                    self.BLOCK_DEVICE_MAPPING_SWAP_SIZE)
+                    self.BLOCK_DEVICE_MAPPING_SWAP_SIZE,
+                    self.BLOCK_DEVICE_MAPPING_EPHEMERAL_SIZE,
+                    self.BLOCK_DEVICE_MAPPING_EPHEMERAL_FORMAT
+                )
 
-            if property_tuple.count(None) == 4:
-                msg = _('Either volume_id, snapshot_id, image_id or '
-                        'swap_size must be specified.')
+            if property_tuple.count(None) == 5:
+                msg = _('Either volume_id, snapshot_id, image_id, swap_size, '
+                        'ephemeral_size or ephemeral_format must be '
+                        'specified.')
                 raise exception.StackValidationFailed(message=msg)
 
             if any((volume_id is not None, snapshot_id is not None,
@@ -1439,10 +1379,20 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                                   network.get(self.NETWORK_PORT) is not None)
             self._validate_network(network)
 
+        # Check if tags is allowed to use
+        if self.properties[self.TAGS]:
+            try:
+                self.client(
+                    version=self.client_plugin().V2_26)
+            except exception.InvalidServiceVersion as ex:
+                msg = _('Cannot use "tags" property - nova does not support '
+                        'it: %s') % six.text_type(ex)
+                raise exception.StackValidationFailed(message=msg)
+
         # retrieve provider's absolute limits if it will be needed
         metadata = self.properties[self.METADATA]
         personality = self.properties[self.PERSONALITY]
-        if metadata is not None or personality:
+        if metadata or personality:
             limits = self.client_plugin().absolute_limits()
 
         # if 'security_groups' present for the server and explicit 'port'
@@ -1455,7 +1405,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         # verify that the number of metadata entries is not greater
         # than the maximum number allowed in the provider's absolute
         # limits
-        if metadata is not None:
+        if metadata:
             msg = _('Instance metadata must not contain greater than %s '
                     'entries.  This is the maximum number allowed by your '
                     'service provider') % limits['maxServerMeta']
@@ -1480,29 +1430,6 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                                         ) if contents is not None else 0,
                                     limits['maxPersonalitySize'], msg)
 
-    def _delete_temp_url(self):
-        object_name = self.data().get('metadata_object_name')
-        if not object_name:
-            return
-        with self.client_plugin('swift').ignore_not_found:
-            container = self.physical_resource_name()
-            swift = self.client('swift')
-            swift.delete_object(container, object_name)
-            headers = swift.head_container(container)
-            if int(headers['x-container-object-count']) == 0:
-                swift.delete_container(container)
-
-    def _delete_queue(self):
-        queue_id = self.data().get('metadata_queue_id')
-        if not queue_id:
-            return
-        client_plugin = self.client_plugin('zaqar')
-        zaqar = client_plugin.create_for_tenant(
-            self.stack.stack_user_project_id, self._user_token())
-        with client_plugin.ignore_not_found:
-            zaqar.queue(queue_id).delete()
-        self.data_delete('metadata_queue_id')
-
     def _delete(self):
         if self.user_data_software_config():
             self._delete_queue()
@@ -1655,6 +1582,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         return defn.freeze(properties=props)
 
     def prepare_for_replace(self):
+        # if the server has not been created yet, do nothing
+        if self.resource_id is None:
+            return
+
         self.prepare_ports_for_replace()
 
     def restore_prev_rsrc(self, convergence=False):
