@@ -606,7 +606,7 @@ class Stack(collections.Mapping):
             stack.update({
                 'action': self.action,
                 'status': self.status,
-                'status_reason': str(self.status_reason)})
+                'status_reason': six.text_type(self.status_reason)})
 
         if only_db:
             stack['parent_resource_name'] = self.parent_resource_name
@@ -949,7 +949,7 @@ class Stack(collections.Mapping):
         if stack is not None:
             values = {'action': self.action,
                       'status': self.status,
-                      'status_reason': str(self.status_reason)}
+                      'status_reason': six.text_type(self.status_reason)}
             self._send_notification_and_add_event()
             if self.convergence:
                 # do things differently for convergence
@@ -981,7 +981,7 @@ class Stack(collections.Mapping):
         if stack is not None:
             values = {'action': self.action,
                       'status': self.status,
-                      'status_reason': str(self.status_reason)}
+                      'status_reason': six.text_type(self.status_reason)}
             self._send_notification_and_add_event()
             stack.persist_state_and_release_lock(self.context, self.id,
                                                  engine_id, values)
@@ -1555,9 +1555,14 @@ class Stack(collections.Mapping):
                 # and new stack resources, we should have user params of both.
                 existing_params.load(newstack.t.env.user_env_as_dict())
                 self.t.env = existing_params
+                # Update the template version, in case new things were used
+                self.t.t[newstack.t.version[0]] = max(
+                    newstack.t.version[1], self.t.version[1])
                 self.t.merge_snippets(newstack.t)
                 self.t.store(self.context)
                 backup_stack.t.env = existing_params
+                backup_stack.t.t[newstack.t.version[0]] = max(
+                    newstack.t.version[1], self.t.version[1])
                 backup_stack.t.merge_snippets(newstack.t)
                 backup_stack.t.store(self.context)
             self.store()
@@ -2094,7 +2099,7 @@ class Stack(collections.Mapping):
         start_time = timeutils.round_to_seconds(self.updated_time or
                                                 self.created_time)
         nowish = timeutils.round_to_seconds(datetime.datetime.utcnow())
-        return (nowish - start_time).seconds
+        return int((nowish - start_time).total_seconds())
 
     def time_remaining(self):
         """Time left before stack times out."""
